@@ -11,8 +11,20 @@ import warnings
 from collections import defaultdict
 from functools import partial
 from functools import wraps
-from typing import (TYPE_CHECKING, Callable, Any, Dict, Generic, Iterable, List,
-                    Optional, Set, Tuple, TypeVar, Union)
+from typing import (
+    TYPE_CHECKING,
+    Callable,
+    Any,
+    Dict,
+    Generic,
+    Iterable,
+    List,
+    Optional,
+    Set,
+    Tuple,
+    TypeVar,
+    Union,
+)
 
 from distutils.errors import DistutilsOptionError, DistutilsFileError
 from packaging.requirements import Requirement, InvalidRequirement
@@ -39,9 +51,7 @@ Target = TypeVar("Target", bound=Union["Distribution", "DistributionMetadata"])
 
 
 def read_configuration(
-    filepath: _Path,
-    find_others=False,
-    ignore_option_errors=False
+    filepath: _Path, find_others=False, ignore_option_errors=False
 ) -> dict:
     """Read given configuration file and returns options from it as a dict.
 
@@ -76,7 +86,8 @@ def apply_configuration(dist: "Distribution", filepath: _Path) -> "Distribution"
 
 
 def _apply(
-    dist: "Distribution", filepath: _Path,
+    dist: "Distribution",
+    filepath: _Path,
     other_files: Iterable[_Path] = (),
     ignore_option_errors: bool = False,
 ) -> Tuple["ConfigHandler", ...]:
@@ -137,7 +148,7 @@ def configuration_to_dict(handlers: Tuple["ConfigHandler", ...]) -> dict:
 def parse_configuration(
     distribution: "Distribution",
     command_options: AllCommandOptions,
-    ignore_option_errors=False
+    ignore_option_errors=False,
 ) -> Tuple["ConfigMetadataHandler", "ConfigOptionsHandler"]:
     """Performs additional parsing of configuration options
     for a distribution.
@@ -240,19 +251,9 @@ class ConfigHandler(Generic[Target]):
         ignore_option_errors,
         ensure_discovered: expand.EnsurePackagesDiscovered,
     ):
-        sections: AllCommandOptions = {}
-
-        section_prefix = self.section_prefix
-        for section_name, section_options in options.items():
-            if not section_name.startswith(section_prefix):
-                continue
-
-            section_name = section_name.replace(section_prefix, '').strip('.')
-            sections[section_name] = section_options
-
         self.ignore_option_errors = ignore_option_errors
         self.target_obj = target_obj
-        self.sections = sections
+        self.sections = dict(self._section_options(options))
         self.set_options: List[str] = []
         self.ensure_discovered = ensure_discovered
         self._referenced_files: Set[str] = set()
@@ -260,6 +261,14 @@ class ConfigHandler(Generic[Target]):
         all files referenced by the "file:" directive. Private API for setuptools only.
         """
 
+    @classmethod
+    def _section_options(cls, options: AllCommandOptions):
+        for full_name, value in options.items():
+            pre, sep, name = full_name.partition(cls.section_prefix)
+            if pre:
+                continue
+            yield name.lstrip('.'), value
+
     @property
     def parsers(self):
         """Metadata item name to parser function mapping."""
@@ -268,40 +277,28 @@ class ConfigHandler(Generic[Target]):
         )
 
     def __setitem__(self, option_name, value):
-        unknown = tuple()
         target_obj = self.target_obj
 
         # Translate alias into real name.
         option_name = self.aliases.get(option_name, option_name)
 
-        current_value = getattr(target_obj, option_name, unknown)
-
-        if current_value is unknown:
+        try:
+            current_value = getattr(target_obj, option_name)
+        except AttributeError:
             raise KeyError(option_name)
 
         if current_value:
             # Already inhabited. Skipping.
             return
 
-        skip_option = False
-        parser = self.parsers.get(option_name)
-        if parser:
-            try:
-                value = parser(value)
-
-            except Exception:
-                skip_option = True
-                if not self.ignore_option_errors:
-                    raise
-
-        if skip_option:
+        try:
+            parsed = self.parsers.get(option_name, lambda x: x)(value)
+        except (Exception,) * self.ignore_option_errors:
             return
 
-        setter = getattr(target_obj, 'set_%s' % option_name, None)
-        if setter is None:
-            setattr(target_obj, option_name, value)
-        else:
-            setter(value)
+        simple_setter = functools.partial(target_obj.__setattr__, option_name)
+        setter = getattr(target_obj, 'set_%s' % option_name, simple_setter)
+        setter(parsed)
 
         self.set_options.append(option_name)
 
@@ -476,7 +473,7 @@ class ConfigHandler(Generic[Target]):
 
         :param dict section_options:
         """
-        for (name, (_, value)) in section_options.items():
+        for name, (_, value) in section_options.items():
             with contextlib.suppress(KeyError):
                 # Keep silent for a new option may appear anytime.
                 self[name] = value
@@ -487,7 +484,6 @@ class ConfigHandler(Generic[Target]):
 
         """
         for section_name, section_options in self.sections.items():
-
             method_postfix = ''
             if section_name:  # [section.option] variant
                 method_postfix = '_%s' % section_name
@@ -524,7 +520,6 @@ class ConfigHandler(Generic[Target]):
 
 
 class ConfigMetadataHandler(ConfigHandler["DistributionMetadata"]):
-
     section_prefix = 'metadata'
 
     aliases = {
@@ -547,7 +542,7 @@ class ConfigMetadataHandler(ConfigHandler["DistributionMetadata"]):
         ignore_option_errors: bool,
         ensure_discovered: expand.EnsurePackagesDiscovered,
         package_dir: Optional[dict] = None,
-        root_dir: _Path = os.curdir
+        root_dir: _Path = os.curdir,
     ):
         super().__init__(target_obj, options, ignore_option_errors, ensure_discovered)
         self.package_dir = package_dir
@@ -615,7 +610,6 @@ class ConfigMetadataHandler(ConfigHandler["DistributionMetadata"]):
 
 
 class ConfigOptionsHandler(ConfigHandler["Distribution"]):
-
     section_prefix = 'options'
 
     def __init__(
@@ -760,7 +754,7 @@ class ConfigOptionsHandler(ConfigHandler["Distribution"]):
         """
         parsed = self._parse_section_to_dict_with_key(
             section_options,
-            lambda k, v: self._parse_requirements_list(f"extras_require[{k}]", v)
+            lambda k, v: self._parse_requirements_list(f"extras_require[{k}]", v),
         )
 
         self['extras_require'] = parsed
